package lekcija5;

import java.util.HashMap; // probati sa LinkedHashMap
import java.io.*;

@SuppressWarnings("serial")
public class XCDStorageMap implements Serializable {
	
	private HashMap<Integer, CDDisk> arhiva;
	
	XCDStorageMap() {
		arhiva = new HashMap<Integer, CDDisk>();
	}
	
	public boolean addCD(CDDisk d) {
		if(arhiva.containsKey(d.getId()))
			return false;
		arhiva.put(d.getId(), new CDDisk(d.getNaziv(), d.getIzvodjac(), d.getId()));
		return true;
	}
	
	public CDDisk removeCD(int id) {
		return arhiva.remove(id);
	}
	
	public CDDisk searchCD(int id) {
		return arhiva.get(id);
	}
	
	public String toString() {
		if(arhiva.isEmpty()) return "U arhivi nema diskova!";
		String retVal = "Sadrzaj CD arhive je:\n";
		for(CDDisk d: arhiva.values())
			retVal += d + "\n";
		return retVal;
	}
	
	public void clear(){
		arhiva.clear();
	}

	public boolean addNewCD(){
		boolean retVal=false;
		String naziv=null, autor=null;
		int id=-1;
			
		BufferedReader in = null;		
		try {
			in = new BufferedReader(new InputStreamReader(System.in));
			System.out.print("Unesite naziv CD-a: ");
			naziv = in.readLine();
			System.out.print("Unesite ime autora: ");
			autor = in.readLine();
			while(id==-1) {
				System.out.print("Unesite identifikacioni broj: ");
				try{
					id = Integer.parseInt(in.readLine());
				} catch(NumberFormatException nfe) {
					System.out.print("Identifikacioni broj mora biti celobrojna vrednost! ");
				}
			}
			retVal = addCD(new CDDisk(naziv,autor,id));
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if(in != null)
				try {
					in.close();
				} catch (Exception e) {
					e.printStackTrace();
				}
		}
		return retVal;
	}

	public void save(String file){
		ObjectOutputStream out = null;
		try {
			out = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(file)));
			out.writeObject(arhiva);
		} catch (Exception e) {
			e.printStackTrace();
        } finally {
            if(out != null)
                try {
                    out.close();
                } catch (Exception e) {
        			e.printStackTrace();
                }
        }
	}
	
	@SuppressWarnings("unchecked")
	public void load(String file){
		ObjectInputStream in = null;
		try {
			in = new ObjectInputStream(new BufferedInputStream(new FileInputStream(file)));
			arhiva = (HashMap<Integer, CDDisk>) in.readObject();
		} catch (Exception e) {
			e.printStackTrace();
        } finally {
            if(in != null)
                try {
                    in.close();
                } catch (Exception e) {
        			e.printStackTrace();
                }
        }
	}

	public void saveText(String file) {
		BufferedWriter out = null;
		try {
			out = new BufferedWriter(new FileWriter(file));
			out.write("CD[ naziv, autor, ID ]");
			out.newLine();
			out.write("----------------------");
			out.newLine();
			for(CDDisk d: arhiva.values()) {
				out.write(d.toString());
				out.newLine();
			}
		} catch (Exception e){
			e.printStackTrace();
        } finally {
            if(out != null)
                try {
                    out.close();
                } catch (Exception e) {
        			e.printStackTrace();
                }
        }
	}
	
	public static void main(String[] args) {
		XCDStorageMap cds = new XCDStorageMap();
		CDDisk d1 = new CDDisk("Stari disk", "Orkestar", 1);
		CDDisk d2 = new CDDisk("Novi disk", "Orkestar", 2);
		
		System.out.println(cds);
		
		cds.addCD(d1);
		cds.addCD(d2);
		cds.addNewCD();
		System.out.println(cds);
		
		CDDisk removed = cds.removeCD(2);
		if(removed != null) 
			System.out.println("Uklonjen disk: " + removed);
		
		CDDisk nadjen = cds.searchCD(1);
		if(nadjen != null) 
			System.out.println("Nadjen disk: " + nadjen);

		System.out.println("*******************************");
		cds.save("cdarhiva.dat");
		System.out.println("Podaci sacuvani u datoteku!");
		System.out.println(cds);

		System.out.println("*******************************");
		cds.clear();
		System.out.println("Podaci obrisani iz arhive!");
		System.out.println(cds);

		System.out.println("*******************************");
		cds.load("cdarhiva.dat");
		System.out.println("Podaci ucitani iz datoteke!");
		System.out.println(cds);

		System.out.println("*******************************");
		cds.saveText("cdarhiva.txt");
		System.out.println("Podaci sacuvani u tekstualnu datoteku!");
		System.out.println(cds);
	}

}
